// Purpose:   Exploring the use of arrays
// Outcome:   byte-array defined blink sequence on pin 13 in 1-second intervals
// Reference: http://stackoverflow.com/questions/28925709/creation-and-addressing-arrays-in-avr-assembly-using-the-atmega8535
// Author:    C. D'Arcy
// Date:      2017 01 29
// Status:    Working
#include "avr/io.h"
arrayStart:                     ; name of the array is simply a label 
.byte 1<<PD5,0,0x20,0b00000000  ; popluate the array with as many all sorts of byte definitions
arrayEnd:                       ; end of the array is another label (defines the size in bytes)
;.word 0xFFFF,0xA0A0            ; (not tested) allocates and initializes array in 2-byte elements
;.space  10                     ; (not tested) reserves the given number of bytes for later use 
.global setup
setup:
  cli                           ;
  sbi _SFR_IO_ADDR(DDRB),DDB5   ; pinMode(13,OUTPUT);
  ldi ZH,hi8(arrayStart)        ; we'll be using indirect addressing
  ldi ZL,lo8(arrayStart)        ;
  ldi XH,hi8(arrayEnd)          ; record the address of the end of the array 
  ldi XL,lo8(arrayEnd)          ;
  clr r16                       ; prepare for Normal Mode
  sts TCCR1A, r16               ; set Normal Mode, now configure the prescaler...
  ldi r16, 0b00000100           ; T1:2^24/2^8/2^16  (prescale) > 1 ovf/s    > 0.5Hz
  sts TCCR1B, r16               ; Timer1 clock = system clock / prescale
//  ldi r16,hi                    ; load TCNT1 (Timer1's 2-byte counter)
//  sts TCNT1H,r16                ; T1:2^16-(2^24/2^8/120)=64989=0xFDDD->120ovf/s=60Hz
//  ldi r16,lo                    ; LED flashing at 24Hz does not seem to produce PoV
//  sts TCNT1L,r16                ; even at 60Hz there appears to be some noticeable flicker
  ldi r16,1<<TOIE1              ; prepare to enable Timer/Counter1 Overflow Interrupt
  sts TIMSK1,r16                ; enable Timer1 Overflow interrupts
  sei                           ; enable global interrupts
ret

.global TIMER1_OVF_vect
TIMER1_OVF_vect:                ; destination for TCNT1 overflow interrupt 
;  in r16,_SFR_IO_ADDR(PORTB)    ; read the port
;  ldi r17,1<<PB5               ; prepare the bit(s) to toggle
  lpm  r16,Z+                   ; array isstored in program memory, not data memory
  out _SFR_IO_ADDR(PORTB),r16   ; update the port
  reti                          ; return from interrupt

.global loop
loop:
  cp ZL,XL                      ; are we at the end of the array?
  brne  loop                    ; No? remain idling
  ldi ZH,hi8(arrayStart)        ; Yes? ring it...
  ldi ZL,lo8(arrayStart)        ;
  rjmp loop                     ; repeat...

